return function ()
    return {
        state = false,
        path = {},
        turnCounter = 0,
        reeledBack = false,
        awake = function(self)
            self.home = {
                x = self.parent.grid.x,
                y = self.parent.grid.y
            }

            self.range = 5

            self.state = game:machine{
                initial = "idle",
                events = {
                    {name = "idle", from = {"chasing", "attacking"}, to = "idle"},
                    {name = "chase", from = "idle", to = "chasing"},
                    {name = "attack", from = "idle", to = "attacking"}
                },
                callbacks = {
                    onchase = function(self, event, from, to, parent)
                        parent.behavior:pathTo("player")
                    end
                }
            }
        end,
        update = function(self, dt, speed)
            if self.parent.stats.health <= 0 then
                self:on_kill()
                game:remove(self.parent)
            end
        end,
        on_kill = function(self)
            if self.parent.parent then
                for index, entity in pairs(self.parent.parent.behavior.spawns) do
                    if entity == self.parent then
                        table.remove(self.parent.parent.behavior.spawns, index)
                    end
                end
            end

            log "The fly falls dead."
        end,
        pathTo = function(self, target)
            if type(target) == "string" then
                local t = game:get(target)
                target = {x = t.grid.x, y = t.grid.y}
            end

            self.parent.grid.collides = false

            local path, success = findPath(self.parent.grid.x, self.parent.grid.y, target.x, target.y, function(check)
                if game.terminal:contains(check.x, check.y) then
                    return not game.terminal:getTileData(check.x, check.y).collides
                else
                    return false
                end
            end)

            self.parent.grid.collides = true

            if path then
                self.path = path
            else
                self.state:idle(self.parent)
            end

        end,
        turn = function(self)
            if self.parent.stats.health > 0 then
                if self[self.state.current] then
                    return self[self.state.current](self)
                end
            else
                return 1
            end
        end,
        idle = function(self)
            if math.random(1, 2) == 1 then
                local targetX, targetY = math.random(1, 3) - 2, math.random(1, 3) - 2

                targetX = self.parent.grid.x + targetX
                targetY = self.parent.grid.y + targetY

                if game.terminal:contains(targetX, targetY) and not tileCollides(targetX, targetY) then
                    move_to(targetX, targetY, MoveType.Absolute)
                else
                    if  game:get("player").grid.x == targetX
                    and game:get("player").grid.y == targetY then
                        game:get("player").behavior:on_hit(self.parent)
                    end
                end
            end

            local player = game:get "player"

            if math.dist(self.parent.grid.x, self.parent.grid.y, player.grid.x, player.grid.y) < self.range then
                self.state:chase(self.parent)
            end

            return 1
        end,
        chasing = function(self)
            local player = game:get "player"

            local recalc = (self._turns % 5 == 0)
                or ((#self.path > 0 ) and tileCollides(self.path[#self.path].x, self.path[#self.path].y))
                or math.dist(player.grid.x, player.grid.y, self.parent.grid.x, self.parent.grid.y) < self.range

            if recalc then
                self:pathTo "player"
            end

            if #self.path > 1 then
                local index = #self.path
                move_to(self.path[index].x, self.path[index].y, MoveType.Absolute)
                table.remove(self.path, index)
            else
                self.state:idle(self.parent)
            end

            for name, dir in pairs(game.definitions.directions) do
                local entities = getEntitiesAt(self.parent.grid.x + dir.x, self.parent.grid.y + dir.y)

                if #entities > 0 then
                    for index, entity in pairs(entities) do
                        if entity.class == "player" then
                            self.state:attack(self.parent)
                        end
                    end
                end
            end

            return 1
        end,
        attacking = function(self)
            if not self.reeledBack then
                local adjacent = false

                for name, dir in pairs(game.definitions.directions) do
                    local entities = getEntitiesAt(self.parent.grid.x + dir.x, self.parent.grid.y + dir.y)

                    if #entities > 0 then
                        for index, entity in pairs(entities) do
                            adjacent = true
                        end
                    end
                end

                if adjacent then
                    self.reeledBack = true
                    log "The fly reels back to attack."
                else
                    self.state:idle(self.parent)
                end
            else
                local hitPlayer = false

                for name, dir in pairs(game.definitions.directions) do
                    local entities = getEntitiesAt(self.parent.grid.x + dir.x, self.parent.grid.y + dir.y)

                    if #entities > 0 then
                        for index, entity in pairs(entities) do
                            if entity.class == "player" then
                                entity.behavior:on_hit(self.parent)
                            end
                            hitPlayer = true

                            log("The fly bites you, you took 1 damage.")
                        end
                    end
                end

                self.reeledBack = false

                if hitPlayer then
                    self.state:idle(self.parent)
                end
            end

            return 1
        end,
        draw = function(self)
            if game.debugMode then
                local ll = game.terminal.currentLayer
                game.terminal:setLayer "debug"
                for index, path in pairs(self.path) do
                    game.terminal:write("@", path.x, path.y, nil, {255, 0, 0}, true)
                end
            end
        end,
        on_hit = function(self, interactor)
            if interactor.class == "player" then
                self.parent.stats.health = self.parent.stats.health - 3
            end
        end
    }
end
