return function()
    return {
        fuse = 10,
        vision = 8,
        flip = 0,
        awake = function(self)
            self.state = game:machine{
                initial = "idle",
                events = {
                    {name = "wake", from = "idle", to = "active"},
                }
            }
        end,

        turn = function(self)
            local turns = 1

            if self[self.state.current] then
                turns = self[self.state.current](self)
            end

            return turns
        end,

        idle = function(self)
            local player = game:get "player"

            if math.random(1, 10) == 1 then
                local move = {
                    x = math.random(1, 3) - 2,
                    y = math.random(1, 3) - 2
                }

                if not tileCollides(self.parent.grid.x + move.x, self.parent.grid.y + move.y) then
                    move_to(
                        move.x, move.y,
                        MoveType.Relative
                    )
                end
            end

            if math.dist(player.grid.x, player.grid.y, self.parent.grid.x, self.parent.grid.y) < self.vision then
                self.state:wake(self.parent)
            end

            return 1
        end,

        active = function(self)
            local player = game:get "player"
            local ghost = self.parent

            local move = {
                x = 0,
                y = 0
            }

            if player.grid.x > ghost.grid.x then
                move.x = 1
            elseif player.grid.x < ghost.grid.x then
                move.x = -1
            end

            if player.grid.y > ghost.grid.y then
                move.y = 1
            elseif player.grid.y < ghost.grid.y then
                move.y = -1
            end

            if move.x ~= 0 or move.y ~= 0 then
                self.fuse = self.fuse - 1

                if self.fuse < 10 then
                    ghost.glyph.char = 'q'
                end

                if self.fuse < 7 then
                    ghost.glyph.char = 'Q'
                end

                if self.fuse < 4 then
                    ghost.glyph.char = 'O'
                end

                ghost.glyph.color = game.terminal:colerp({255, 0, 0}, {255, 255, 0}, math.max(self.fuse, 1) / 10)

                if self.fuse <= 0 then
                    explode(ghost.grid.x, ghost.grid.y, 5)
                    game:remove(ghost)
                end
            end

            move_to(move.x, move.y, MoveType.Relative)

            return math.floor(math.lerp(1, 3, math.max(self.fuse, 1) / 10))
        end,

        update = function(self, dt, speed)
            self.flip = self.flip * (dt * speed)

            if self.fuse < 2 then
                if self.flip % 0.5 then
                    self.parent.glyph.color = {255, 255, 0}
                else
                    self.parent.glyph.color = {255, 0, 0}
                end
            end

        end,

        remove = function(self)
            game.locked = false
        end
    }
end
