--
--
--
--
--

return function()
	return {
		state = {},
		consecutiveMoves = 0,
		idleTime = 0,
		command = '',
		submitCommand = false,
		awake = function(self)
			self.state = game:machine{
				initial = "idle",
				events = {
					{name = "command", from = "idle", to = "locked"},
					{name = "unlock", from = "locked", to = "idle"},
					{name = "idle", from = "selecting", to = "idle"},
					{name = "select", from = "idle", to = "selecting"}
				},
				callbacks = {
					onselect = function(self, event, from, to, parent, callback)
						game:add("cursor", {
							parent = parent,
							grid = {
								x = parent.grid.x,
								y = parent.grid.y
							}
						})

						game.promise:promise("cursor", {
							onResolve = callback,
							parent = parent
						})
					end
				}
			}
		end,
		update = function(self, dt, speed)
			if self.parent.stats.health <= 0
			or self.parent.stats.battery <= 0 then
				script "lose_game"
			end

			if not game.locked then
				if self[self.state.current] then
					self[self.state.current](self, dt, speed)
				end
			end
		end,
		locked = function(self, dt, speed)
			if self.submitCommand then
				game.commandProcessor:process(self.command, self.parent)

				self.submitCommand = false
				self.command = ''

				self.state:idle(self.parent)
			end
		end,
		idle = function(self, dt, speed)
			if self.parent.queue:isEmpty() then
				local doneMove = false
				local ms = math.clamp(0.5 / self.consecutiveMoves, 0.1, 0.5)

				local target = {0, 0}

				if game.controls:pressed "up" then
					target = game.definitions.directions.up
					self.dir = "up"
					doneMove = true
				elseif game.controls:pressed "down" then
					target = game.definitions.directions.down
					self.dir = "down"
					doneMove = true
				elseif game.controls:pressed "left" then
					target = game.definitions.directions.left
					self.dir = "left"
					doneMove = true
				elseif game.controls:pressed "right" then
					target = game.definitions.directions.right
					self.dir = "right"
					doneMove = true
				end

				if doneMove then
					addTurns(1)
					local doMove = true

					if tileCollides(self.parent.grid.x + target.x, self.parent.grid.y + target.y) then
						doMove = false

						local entities = getEntitiesAt(self.parent.grid.x + target.x, self.parent.grid.y + target.y)

						if #entities > 0 then
							for index, entity in pairs(entities) do
								entity.behavior:on_hit(self.parent)
							end
						end
					end

					if doMove then
						move_to(target.x, target.y, nil, 0.01)
						self.consecutiveMoves = self.consecutiveMoves + 1
					end
				else
					self.consecutiveMoves = 1
				end

				if game.controls:pressed "b" and not doneMove then
					self.state:select(self.parent, function(self)
						local cursor = game:get "cursor"
						local entities = {}

						for index, entity in pairs(getEntitiesAt(cursor.grid.x, cursor.grid.y)) do
							if entity.behavior then
								if entity ~= cursor then
									table.insert(entities, {
										entity = entity,
										label = entity.stats.name,
										onclick = function(self)
											self.entity.behavior:examine()
										end
									})
								end
							end
						end

						if #entities > 1 then
							terminal("Select something to examine:", entities)
						elseif #entities == 1 then
							entities[1].entity.behavior:examine()
						else
							log "Nothing there..."
						end
					end)
				end

				if not doneMove then
					self.idleTime = self.idleTime + (dt * speed)
				else
					self.idleTime = 0
				end
			end
		end,
		turn = function(self)
			self.parent.stats.battery = self.parent.stats.battery - self.parent.stats.decay

			return 1
		end,
		onExplode = function(self, x, y, radius)
			self.parent.stats.health = self.parent.stats.health - 1
			log "You get clipped in the explosion."
		end,
		draw = function(self)
			game.terminal:setLayer "ui"

			if self.parent.stats.health ~= self.parent.stats.maxHealth then
				game.terminal:print("HP", 2, 3, nil, nil, true)
				for h = 1, self.parent.stats.health do
					game.terminal:print(string.char(3), 4 + h, 3, {255, 0, 0}, {0, 0, 0}, true)
				end
			end

			if self.parent.stats.battery ~= self.parent.stats.maxBattery then
				game.terminal:print("DC", 2, 4, self.parent.stats.battery < 1 and {255, 0, 0} or nil, {0,0,0}, true)
				for h = 0, self.parent.stats.maxBattery do
					local visibility = 255

					if self.parent.stats.battery - h <= 1 then
						visibility = math.lerp(0, 255, (self.parent.stats.battery - h / 1))
					end

					game.terminal:print(string.char(3), 5 + h, 4, {0, 255, 255, visibility}, {0, 0, 0}, true)
				end
			end

			if self.idleTime > 3 or game.terminal.lastLog < 3 then
				for i = 1, math.min(#game.terminal.log, 5) do
					game.terminal:print(game.terminal.log[i],
						game.terminal.width - #game.terminal.log[i],
						(self.parent.grid.y > (game.terminal.height / 2)) and 5 - i or game.terminal.height - i,
						{255, 255, 0}, {0, 0, 0},
					true)
				end
			end

			if self.state.current == "locked" then
				game.terminal:print("> " .. self.command,
					self.parent.grid.x,
					self.parent.grid.y + 2, nil, {0,0,0}, true)
			end
		end
	}
end
