return function()
    return {
        text = "No text provided.",
        width = 50,
        height = 8,

        choices = {},
        selector = 1,
        padding = 3,
        removeOnConfirm = true,
        awake = function(self)

            if #self.text > self.width - (self.padding * 2) then
                local lines = {}
                local words = self.text:separate(' ')
                local currentLine = ''

                while #words > 0 do
                    if #(currentLine .. ' ' .. words[1]) > self.width - (self.padding * 2) then
                        table.insert(lines, currentLine)
                        currentLine = ''
                    else
                        currentLine = currentLine .. ' ' .. words[1]
                        table.remove(words, 1)
                    end
                end

                table.insert(lines, currentLine)

                self.text = lines

                self.height = self.height + #lines
            end

            game.terminal:setLayer "persistent_ui"
            for x = 1, self.width do
                for y = 1, self.height + (#self.choices > 0 and #self.choices + self.padding or 0) do
                    if x == 1 or x == self.width
                    or y == 1 or y == self.height + (#self.choices > 0 and #self.choices + self.padding or 0) then
                        game.terminal:print("#", self.parent.x + x, self.parent.y + y, {255, 255, 255}, {255, 255, 255})
                    else
                        game.terminal:print("#", self.parent.x + x, self.parent.y + y, {0,0,0}, {0,0,0})
                    end
                end
            end

            if type(self.text) == "string" then
                game.terminal:print(self.text, self.parent.x + 3, self.parent.y + self.padding, {255, 255, 255}, {0, 0, 0})
            else
                for index, line in pairs(self.text) do
                    game.terminal:print(line, self.parent.x + 3, self.parent.y + self.padding + index, {255, 255, 255}, {0, 0, 0})
                end
            end

            if #self.choices > 0 then
                for y = 1, #self.choices do
                    local choice = self.choices[y]
                    game.terminal:print(choice.label, self.parent.x + 4, self.parent.y + self.height - #self.choices + y, {255, 255, 255}, {0, 0, 0})
                end

                game.terminal:print("[Press X to select]", self.parent.x + self.padding, self.parent.y + self.height + (#self.choices > 0 and #self.choices + self.padding or 0) , {255, 255, 255}, {0, 0, 0})
            else
                game.terminal:print("[Press X to close]", self.parent.x + self.padding, self.parent.y + self.height + (#self.choices > 0 and #self.choices + self.padding or 0) , {255, 255, 255}, {0, 0, 0})
            end

            game.locked = true
        end,
        update = function(self, dt, speed)
            game.locked = true

            if #self.choices > 0 then
                if game.controls:pressed "up" then
                    self.selector = self.selector - 1
                elseif game.controls:pressed "down" then
                    self.selector = self.selector + 1
                end

                self.selector = math.clamp(self.selector, 1, #self.choices, true)

                if game.controls:pressed "x" then
                    self.choices[self.selector].onclick(self.choices[self.selector])

                    if self.removeOnConfirm then
                        game:remove(self.parent)
                    end
                end
            else
                if game.controls:pressed "x" then
                    if self.removeOnConfirm then
                        game:remove(self.parent)
                    end
                end
            end
        end,
        draw = function(self)
            if #self.choices > 0 then
                game.terminal:setLayer "ui"
                game.terminal:print(">", self.parent.x + 2, self.parent.y + self.height - #self.choices + self.selector, {255, 255, 0}, {0, 0, 0}, true)
            end
        end,
        remove = function(self)
            game.terminal:setLayer "persistent_ui"
            game.terminal:clear()
            game.locked = false
        end

    }
end
