--
--
--
--
--

return function(map, rooms)
    game.terminal:clear("structures")

    for index, entity in pairs(game:getEntities()) do
        if entity.class ~= "player" then
            game:remove(entity)
        end
    end

    -- Wall generation.
    for y = 1, #map do
        map[y][1] = 2
        map[y][#map[1] - 1] = 2
    end

    for x = 1, #map[1] do
        map[1][x] = 2
        map[#map - 1][x] = 2
    end

    for y = 1, #map do
        for x = 1, #map[1] do

            if map[y][x] == 2 then
                local isWall = false

                for index, dir in pairs(game.definitions.directions) do
                    local checkX = x + dir.x
                    local checkY = y + dir.y

                    if game.terminal:contains(checkX, checkY) then
                        if map[checkY] and map[checkY][checkX] then
                            if map[checkY][checkX] == 1 then
                                isWall = true
                            end
                        end
                    end
                end

                if isWall then
                    print "Preparing wall."
                    map[y][x] = 3
                end
            end
        end
    end

    local roomList = {1}

    while #roomList < #rooms do
        table.insert(roomList, weightedRandom{
            [2] = 20,
            [3] = 5,
            [4] = 8
        })

        if #roomList == #rooms - 1 then
            table.insert(roomList, 6)
        end
    end

    for r = 1, #rooms do
        room = rooms[r]

        if roomList[r] == 1 then
            if state "depth" == 1 then
                drawStructure("start", room.x - 7, room.y - 2)
            else
                local lc = game.context
                game.context = game:get "player"

                move_to(room.x + math.random(1, room.w), room.y + math.random(1, room.h), MoveType.Absolute)

                game.context = lc
            end
        end

        if roomList[r]  == 6 then
            game:add("props/stairs", {
                grid = {
                    x = room.x + math.random(1, room.w),
                    y = room.y + math.random(1, room.h)
                }
            })
        end

        if roomList[r] == 2 then
            for y = room.y, room.y + room.h do
                for x = room.x, room.x + room.w do
                    if map[y][x] == 1 then
                        if math.random(1, 10) == 1 then
                            local wallAdjacent = false

                            if x == room.x or x == room.x + room.w
                            or y == room.y or y == room.y + room.h then
                                wallAdjacent = true
                            end

                            if not wallAdjacent then
                                if math.random(1, 2) == 1 then
                                    map[y][x] = 4
                                else
                                    game:add("props/mushroom", {
                                        grid = {x = x, y = y}
                                    })
                                end
                            else
                                game:add("props/mushroom", {
                                    grid = {x = x, y = y}
                                })
                            end
                        else
                            if math.random(1, 2) == 1 then
                                map[y][x] = 5
                            end
                        end
                    end
                end
            end

            for z = 1, math.random(1, 4) do

                local spawn = weightedRandom{
                    [1] = 10,
                    [2] = 5,
                    [3] = 2,
                    [4] = 1
                }

                if spawn == 1 then
                    game:add("mobs/zombie", {
                        grid = {
                            x = math.random(room.x, room.x + room.w),
                            y = math.random(room.y, room.y + room.h)
                        }
                    })
                elseif spawn == 2 then
                    game:add("mobs/fly", {
                        grid = {
                            x = math.random(room.x, room.x + room.w),
                            y = math.random(room.y, room.y + room.h)
                        }
                    })
                elseif spawn == 3 then
                    game:add("mobs/big_boy", {
                        grid = {
                            x = math.random(room.x, room.x + room.w),
                            y = math.random(room.y, room.y + room.h)
                        }
                    })
                end
            end

        end

        if roomList[r] == 3 then
            for x = room.x, room.x + room.w do
                for y = room.y, room.y + room.h do
                    if x == room.x or x == room.x + room.w
                    or y == room.y or y == room.y + room.h then
                        map[y][x] = 8
                    else
                        map[y][x] = 9
                    end
                end
            end

            local added = {}

            if room.w > 6 then
                buildStructure("tank", room.x + 1, room.y, function(c, x, y, fg, bg)
                    return 7
                end)
                table.insert(added, game:add("mobs/specimen", {
                    grid = {
                        x = room.x + 2,
                        y = room.y + 2
                    }
                }))

                buildStructure("tank", room.x + room.w - 3, room.y, function(c, x, y, fg, bg)
                    return 7
                end)
                table.insert(added, game:add("mobs/specimen", {
                    grid = {
                        x = room.x + room.w - 2,
                        y = room.y + 2
                    }
                }))

                if math.random(1, 2) == 1 then
                    game:add("battery", {
                        grid = {
                            x = math.random(room.x + 3, room.x + room.w - 3),
                            y = room.y + 1
                        }
                    })
                end
            else
                buildStructure("tank", room.x + math.floor(room.w / 2), room.y + 1, function(c, x, y, fg, bg)
                    return 7
                end)
            end

            game:add({
                class = "computer",
                entities = added,
                stats = {
                    name = "computer",
                    flavor = "A computer, its CRT emits a quiet hum..."
                },
				fov = {
					type = FOVType.VISION,
					radius = 5,
					color = {
						r = 0,
						g = 255,
						b = 255,
						a = 255
					}
				},
                behavior = {
                    done = false,
                    on_hit = function(self)
                        if not self.done then
                            terminal(
                                f("There's a big red button on this desk, and a text that reads: [%i]. Pay 1W and press this button?", #self.parent.entities),
                                {
                                    {
                                        computer = self.parent,
                                        label = "YES!",
                                        onclick = function(self)
                                            local computer = self.computer

                                            if #computer.entities > 0 then
                                                local entity = computer.entities[#computer.entities]
                                                entity.behavior.state:free(entity)
                                                table.remove(computer.entities, #computer.entities)

                                                game:get("player").stats.battery = game:get("player").stats.battery - 1

                                                if #computer.entities == 0 then
                                                    computer.behavior.done = true
                                                end
                                            end
                                        end
                                    },
                                    {
                                        computer = self.parent,
                                        label = "No.",
                                        onclick = function(self)
                                            terminal "Your restraint is impressive."
                                        end
                                    }
                                }
                            )
                        else
                            terminal "Your lack of restraint is keeping this big red button down."
                        end
                    end
                },
                grid = {
                    x = room.x + math.floor(room.w / 2),
                    y = room.y + room.h - 2,
                    collides = true
                },
                glyph = {
                    char = '#',
                    color = {255, 255, 0},
                    background = {0, 0, 0, 0}
                }
            })
        end

        if roomList[r] == 4 then
            for x = room.x, room.x + room.w do
                for y = room.y, room.y + room.h do
                    if x == room.x or x == room.x + room.w
                    or y == room.y or y == room.y + room.h then
                        map[y][x] = 8
                    else
                        map[y][x] = 9
                    end
                end
            end

            buildStructure("light", room.x + 1, room.y + 1, 10)
            buildStructure("light", room.x + room.w - 2, room.y + 1, 10)
            buildStructure("light", room.x + room.w - 2, room.y + room.h - 2, 10)
            buildStructure("light", room.x + 1, room.y + room.h - 2, 10)

            if math.random(1, 10) == 1 then
                game:add("props/cat", {
                    grid = {
                        x = room.x + math.random(1, room.w - 1),
                        y = room.y + math.random(1, room.h - 1)
                    }
                })
            else
                game:add("mobs/ghost", {
                    grid = {
                        x = room.x + math.floor(room.w / 2),
                        y = room.y + math.floor(room.h / 2)
                    }
                })
            end
        end
    end

    return map
end
