--
--
--
--
--

local PromiseTracker = {
    promises = {}
}

function PromiseTracker:promise(name, options)
	if not options then
		return self.promises[name]
	else
		local options = options and options or {}
		options.name = name

		self.promises[name] = options
	end
end

function PromiseTracker:update(dt, speed)
	for name, promise in pairs(self.promises) do
		if promise.onUpdate then
			if type(promise.onUpdate) == "string" then
				game.scripts[promise.onUpdate](promise, dt, speed)
			end

			if type(promise.onUpdate) == "table" then
				table.recurse(game.scripts, promise.onUpdate)(promise, dt, speed)
			end

			if type(promise.onUpdate) == "function" then
				promise:onUpdate(dt, speed)
			end
		end
	end
end

function PromiseTracker:draw()
	for name, promise in pairs(self.promises) do
		if promise.onDraw then
			if type(promise.onDraw) == "function" then
				promise:onDraw(dt, speed)
			end
		end
	end
end

function PromiseTracker:resolve(name)
	if self:exists(name) then
		if self.promises[name].onResolve then
			if type(self.promises[name].onResolve) == "string" then
				game.scripts[self.promises[name].onResolve]()
			end

			if type(self.promises[name].onResolve) == "table" then
				table.recurse(game.scripts, self.promises[name].onResolve)()
			end

			if type(self.promises[name].onResolve) == "function" then
				self.promises[name]:onResolve()
			end
		end
	end

	self.promises[name] = nil
end

function PromiseTracker:close(name)
	self.promises[name] = nil
end

function PromiseTracker:exists(name)
	return self.promises[name] ~= nil
end

function PromiseTracker:set(name, key, value)
	if self:exists(name) then
		self.promises[name][key] = value
	else
		return false
	end
end

function PromiseTracker:get(name, key)
	if self:exists(name) then
		if not key then
			return self.promises[name]
		else
			if not self.promises[name][key] then
				return false
			else
				return self.promises[name][key]
			end
		end
	else
		return false
	end
end

return PromiseTracker
