--
--
--
--
--

return function()
	local behavior = game.system()

	behavior.turns = 0

	function behavior:validate(entity, templating)
		printf("loading entity %s", entity.name)
		if entity.behavior then
			if type(entity.behavior) == "string" then
				entity.behavior = {name = entity.behavior}
			end

			if entity.behavior.scheme then
				entity.behavior.name = "default"
				for key, value in pairs(entity.behavior.scheme) do
					if type(value) == "string" then
						entity.behavior[key] = require(string.format("assets/behaviors/%s", value))
					else
						entity.behavior[key] = value
					end
				end
			end

			entity.behavior = table.merge(
				table.merge(
					require('game.components.behavior')(),
					entity.behavior.name and require(string.format('assets.behaviors.%s', entity.behavior.name))() or {}
				),
				entity.behavior
			)

			entity.behavior.parent = entity
			entity.behavior._turns = self.turns

			if not templating then
				entity.behavior:awake()
				entity.behavior._awake = true
			end
		end

		return entity
	end

	function behavior:filter(entity, validating)
		if validating then
			return entity.behavior
		else
			return entity.behavior and entity.behavior:validState(game.state)
		end
	end

	function behavior:process(entity, dt, speed)
		game.context = entity

		if entity.behavior then
			if not entity.behavior._awake then
				entity.behavior:awake()
				entity.behavior._awake = true
			end

			if entity.behavior.holdForQueue then
				if areQueuesEmpty({entity.name}) then
					entity.behavior:update(dt, speed)
				end
			else
				entity.behavior:update(dt, speed)
			end

			while entity.behavior._turns < self.turns do
				local turns = entity.behavior:turn()

				if not turns then
					error(f("You forgot to assign a turn timer to this, idiot: %s", entity.behavior.name)) 
				end

				entity.behavior._turns = entity.behavior._turns + turns
			end
		end

	end

	function behavior:render(entity)
		entity.behavior:draw()
	end

	function behavior:remove(entity)
		entity.behavior:remove()
	end

	return behavior
end
