return function()
    local fov = game.system()

    fov.visible = {}
	fov.light = {}

    function fov:filter(entity)
        return entity.grid and entity.fov
    end

    function fov:validate(entity)
        entity.fov = table.merge(
            require('game.components.fov')(),
            entity.fov
        )

        return entity
    end

    function fov:update(dt, speed)
        fov.visible = {}
		fov.light = {}
    end

    function fov:process(entity, dt, speed)
		if entity.fov.type == FOVType.LIGHT or entity.fov.type == FOVType.BOTH then
			if entity.fov.color then
				get_fov(entity.grid.x, entity.grid.y, entity.fov.radius, function(x, y, sx, sy, radius, options)
					local light = game:getSystem('fov').light

					local targetColor = options.color
					targetColor.a = 255 - ((math.dist(x, y, sx, sy) / radius) * 255)

					if light[f("%i,%i", x, y)] then
						targetColor = game.terminal:colerp(light[f("%i,%i", x, y)].color, targetColor, 0.5)
					end

					game:getSystem('fov').light[f("%i,%i", x, y)] = {
						x = x,
						y = y,
						color = targetColor
					}
				end, {color = deepcopy(entity.fov.color)})
			else
				error(f("Entity %s has no color attribute.", entity.name))
			end
		elseif entity.fov.type == FOVType.VISION or entity.fov.type == FOVType.BOTH then
	        get_fov(entity.grid.x, entity.grid.y, entity.fov.radius, function(x, y, sx, sy, radius)
	            game:getSystem('fov').visible[f("%i,%i", x, y)] = {
	                x = x,
	                y = y,
					vision = (math.dist(x, y, sx, sy) / radius)
	            }
	        end)
		end
    end

    function fov:draw()
        game.terminal:setLayer "fov"

        local lc = love.graphics.getCanvas()
        love.graphics.setCanvas(game.terminal.displays.fov.canvas)
        love.graphics.setColor(0, 0, 0)
        love.graphics.rectangle("fill", 0, 0, game.width, game.height)

        for index, visible in pairs(self.visible) do
        	love.graphics.setColor(255, 255, 255)
            love.graphics.setScissor((visible.x - 1) * 12, (visible.y - 1) * 12, 12, 12)
            love.graphics.clear(0, 0, 0, math.cerp(0, 255, visible.vision))
			love.graphics.setScissor()
        end

        love.graphics.setScissor()

        local lc = love.graphics.getCanvas()
        love.graphics.setCanvas(game.terminal.displays.lights.canvas)
        love.graphics.clear()

        for index, visible in pairs(self.light) do
			local color = visible.color

			if color then
				love.graphics.setColor(visible.color.r, visible.color.g, visible.color.b, visible.color.a)
	            love.graphics.rectangle("fill", (visible.x - 1) * 12, (visible.y - 1) * 12, 12, 12)
			end
        end

		love.graphics.setColor(255, 255, 255, 255)
        love.graphics.setCanvas(lc)
    end

    return fov
end
