--
--
--
--
--

return function()
	local interface = game.system()

	function interface:validate(entity)
		entity.interface = table.merge(require("game.components.interface")(), entity.interface)

		entity.interface.options.id = entity.name
		entity.interface.options.font = game.fonts[entity.interface.options.font]

		if entity.width then
			entity.interface.width = entity.width
			entity.interface.options.width = entity.width
		end

		if entity.height then
			entity.interface.height = entity.height
			entity.interface.options.height = entity.height
		end

		if (entity.interface.type == "Label" or entity.interface.type == "Button") and entity.label then
			entity.interface.state = entity.label
		end

		if entity.interface.type == "List" then
			entity.interface.state = {
				entries = {},
				position = 0,
				highlighted = 0
			}
		end

		if entity.interface.type == "Stepper" and entity.steps then
			entity.interface.state.steps = entity.steps:separate(";")
		end

		if entity.interface.type == "Button" and entity.onclick then
			entity.interface.calls.hit = entity.onclick
		end

		if entity.interface.calls.init then
			if type(entity.interface.calls.init) == "string" then
				loadstring(entity.interface.calls.init)(entity)
			else
				entity.interface.calls.init(entity)
			end
		end

		return entity
	end

	function interface:filter(entity, isValidating)
		if entity ~= nil then
			if isValidating then
				return entity.interface
			else
				return entity.interface and (not entity.interface.tab or entity.interface.tab == game.tab)
			end
		end
	end

	function interface:update(dt, speed)
		game.suit.layout:reset()
	end

	function interface:process(entity, dt, speed)
		if entity.interface.type == "List" then
			self:processList(entity, dt, speed)
			return true
		end

		if entity.interface.type == "Stepper" then
			self:processStepper(entity, dt, speed)
			return true
		end

		local ui = game.suit[entity.interface.type](
			entity.interface.type == "ImageButton" and entity.interface.state.normal or entity.interface.state,
			entity.interface.options,
			(entity.x or 0) + (entity.interface.x or 0),
			(entity.y or 0) + (entity.interface.y or 0),
			entity.interface.width,
			entity.interface.height
		)

		if entity.interface.calls.update then
			entity.interface.calls.update(entity, dt, speed)
		end

		for tag, call in pairs(entity.interface.calls) do
			if call and ui[tag] then
				if type(call) == "string" then
					local call = loadstring(call)
					if call then
						call(entity)
					end
				else
					call(entity)
				end
			end
		end
	end

	function interface:processList(entity, dt, speed)
		entity.interface.state.position = math.clamp(entity.interface.state.position, 0, #entity.interface.state.entries)

		if #entity.interface.buttons == 0 then
			table.insert(entity.interface.buttons, game:add('ui/textbox', {
				parent = entity,
				x = entity.x,
				y = entity.y,
				width = math.floor(entity.width * 0.7),
				height = 16,
				interface = {
					tab = entity.interface.tab,
					state = {
						text = ""
					},
					calls = {
						submitted = function(self)
							table.insert(self.parent.interface.state.entries, self.interface.state.text)
						end
					}
				}
			}))
			local textbox = entity.interface.buttons[1]
			table.insert(entity.interface.buttons, game:add('ui/button', {
				x = entity.x + (entity.width * 0.7),
				y = entity.y,
				width = math.floor(entity.width * 0.3),
				height = 16,
				parent = entity,
				textbox = textbox,
				interface = {
					tab = entity.interface.tab,
					state = "ADD",
					calls = {
						hit = function(self)
							if self.textbox.interface.state.text ~= "" then
								table.insert(self.parent.interface.state.entries, self.textbox.interface.state.text)
								self.textbox.interface.state.text = ""
							end
						end
					}
				}
			}))

			table.insert(entity.interface.buttons, game:add('ui/button', {
				x = entity.x,
				y = entity.y + 16,
				width = math.floor(entity.width / 2),
				height = 16,
				parent = entity,
				interface = {
					tab = entity.interface.tab,
					state = "UP",
					calls = {
						hit = function(self)
							self.parent.interface.state.position = math.clamp(self.parent.interface.state.position - 1, 0, #self.parent.interface.state.entries)
						end
					}
				}
			}))

			table.insert(entity.interface.buttons, game:add('ui/button', {
				parent = entity,
				x = entity.x + math.floor(entity.width / 2),
				y = entity.y + 16,
				width = math.floor(entity.width / 2),
				height = 16,
				interface = {
					state = "DOWN",
					tab = entity.interface.tab,
					calls = {
						hit = function(self)
							self.parent.interface.state.position = math.clamp(self.parent.interface.state.position + 1, 0, #self.parent.interface.state.entries)
						end
					}
				}
			}))
		end

		local iters = math.floor((entity.height - 32) / 8)
		for i = 1, iters do
			if game.mouse.x > entity.x
			and game.mouse.x < entity.x + entity.width then
				if game.mouse.y > entity.y + (i * 8) + 32
				and game.mouse.y < entity.y + ((i + 1) * 8) + 32 then
					if #entity.interface.state.entries >= entity.interface.state.position + i then
						entity.interface.state.highlighted = entity.interface.state.position + i

						if game.mouse.pressed then
							table.remove(entity.interface.state.entries, entity.interface.state.position + i)
						end
					else
						entity.interface.state.highlighted = false
					end
				end
			end
		end
	end

	function interface:processStepper(entity, dt, speed)
		if #entity.interface.children == 0 then
			print "Loading stepper childrne"
			table.insert(entity.interface.children, game:add("ui/label", {
				parent = entity,
				x = entity.x + 16,
				y = entity.y,
				width = entity.width - 32,
				label = "no steps",
				interface = {
					tab = entity.interface.tab,
					state = "no steps",
					options = {
					},
					calls = {
						update = function(self)
							self.interface.state = tostring(self.parent.interface.state.steps[self.parent.interface.state.position])
						end
					}
				}
			}))

			table.insert(entity.interface.children, game:add("ui/button", {
				parent = entity,
				x = entity.x,
				y = entity.y,
				width = 16,
				height = entity.height,
				interface = {
					tab = entity.interface.tab,
					state = "-",
					calls = {
						hit = function(self)
							self.parent.interface.state.position = math.clamp(self.parent.interface.state.position-1, 1, #self.parent.interface.state.steps)
						end
					}
				}
			}))

			table.insert(entity.interface.children, game:add("ui/button", {
				parent = entity,
				x = entity.x + entity.width - 16,
				y = entity.y,
				width = 16,
				height = entity.height,
				interface = {
					tab = entity.interface.tab,
					state = "+",
					calls = {
						hit = function(self)
							self.parent.interface.state.position = math.clamp(self.parent.interface.state.position+1, 1, #self.parent.interface.state.steps)
						end
					}
				}
			}))
		end
	end

	function interface:render(entity)
		if entity.interface.type == "List" then
			self:renderList(entity)
		end
	end

	function interface:renderList(entity)
		local lc = love.graphics.getCanvas()
		love.graphics.setCanvas(game.layers.ui)

		for i = 1, math.floor((entity.height - 32) / 8) do
			if entity.interface.state.entries[entity.interface.state.position + i] then
				if entity.interface.state.position + i == entity.interface.state.highlighted then
					love.graphics.setColor(55, 55, 55)
					love.graphics.rectangle("fill", entity.x + 8, entity.y + 30 + (i * 8), entity.width - 8, 10)
					love.graphics.setColor(255, 255, 255)
				end

				love.graphics.print(entity.interface.state.entries[entity.interface.state.position + i], entity.x + 8, entity.y + 32 + (i * 8))
			end
		end

		love.graphics.rectangle("fill", entity.x, entity.y, 4, entity.height)
		love.graphics.setColor(125, 125, 0)
		love.graphics.rectangle("fill", entity.x, entity.y + math.lerp(0, entity.height, entity.interface.state.position / #entity.interface.state.entries), 4, 8)
		love.graphics.setColor(255, 255, 255)

		love.graphics.setCanvas(lc)
	end

	function interface:remove(entity)
		if entity.interface.type == "List" then
			for index, button in pairs(entity.interface.buttons) do
				game:remove(button)
			end
		end

		if entity.interface.type == "Stepper" then
			for index, child in pairs(entity.interface.children) do
				game:remove(child)
			end
		end
	end

	function interface:draw()
        local lc = love.graphics.getCanvas()
        love.graphics.setCanvas(game.layers.suit)
		love.graphics.setColor(255, 255, 255)
		local lf = love.graphics.getFont()
        game.suit.draw()

		love.graphics.setFont(lf)
        love.graphics.setCanvas(lc)
    end

	return interface
end
