--
--
--
--
--

return function()

	local tilemap = game.system()
	tilemap.drawOrder = {}
	tilemap.rerenderAll = false

	function tilemap:validate(entity)
		entity.map = table.merge(require('game.components.map')(), entity.map)
		return entity
	end

	function tilemap:filter(entity)
		if entity ~= nil then
			return entity.map
		end
	end

	function tilemap:update()
		self.drawOrder = {}
	end

	function tilemap:process(entity, dt, speed)
		for index, tile in pairs(entity.map.animatedTiles) do
			tile.age = tile.age + (dt * speed)

			if tile.age > tile.animation[tile.index].life then
				while tile.age > tile.animation[tile.index].life do
					tile.age = tile.age - tile.animation[tile.index].life
					tile.index = tile.index + 1

					if tile.index > #tile.animation then
						tile.index = 1
					end
				end
			end

			if tile.index > #tile.animation then
				tile.index = 1
			end
		end
	end

	function tilemap:draw()
		if self.rerenderAll then
			for name, entity in pairs(game:getEntities()) do
				if self:filter(entity) then
					self:renderFullTilemap(entity)
				end
			end

			self.rerenderAll = false
		end
	end

	function tilemap:renderFullTilemap(entity)
		local lc = love.graphics.getCanvas()
		love.graphics.setCanvas(entity.map.canvas)

		if entity.map.version < 1.2 then
			for y = 1, entity.map.height do
				for x = 1, entity.map.width do
					if entity.map.tiles[y][x] ~= 0 then
						love.graphics.draw(
							entity.map.tilesheet,
							entity.map.quads[entity.map.tiles[y][x]],
							(x - 1) * map.tilewidth,
							(y - 1) * map.tileheight
						)
					end
				end
			end
		else
			for y = 1, entity.map.height do
				for x = 1, entity.map.width do
					local tile = entity.map.tiles[y][x]
					local tileset = entity.map.parent.tilesets[1]
					local offset = 1

					for first, t in pairs(entity.map.parent.tilesets) do
						if tile >= first then
							offset = first
							tileset = t
						end
					end

					tile = tile - offset

					if tile >= 0 then
						if tileset.quads[tile] then
							love.graphics.setScissor((x - 1) * tileset.tileWidth, (y - 1) * tileset.tileHeight, tileset.tileWidth, tileset.tileHeight)
							love.graphics.clear()
							love.graphics.draw(tileset.sheet, tileset.quads[tile], (x - 1) * tileset.tileWidth, (y - 1) * tileset.tileHeight)
							love.graphics.setScissor()
						else
							printf("Tile quad not found: %s:%i", tileset.name, tile)
						end
					end
				end
			end
		end

		love.graphics.setCanvas(lc)
	end

	function tilemap:render(entity)
		if entity.map.visible then
			if entity.map.version < 1.2 then
				self:renderVersion1(entity)
			else
				self:renderVersion1_2(entity)
			end
		end
	end

	function tilemap:renderVersion1(entity)
		local lc = love.graphics.getCanvas()
		love.graphics.setCanvas(entity.map.canvas)

		for index, tile in pairs(entity.map.animatedTiles) do

			love.graphics.draw(entity.map.tilesheet,
				entity.map.quads[tile.animation[tile.index].quad + 1],
				(tile.x - 1) * entity.map.tile.width,
				(tile.y - 1) * entity.map.tile.height
			)
		end

		for index, tile in pairs(entity.map.changedTiles) do
			entity.map.tiles[tile.y][tile.x] = tile.id

			love.graphics.draw(entity.map.tilesheet,
				entity.map.quads[tile.id + 1],
				(tile.x - 1) * entity.map.tile.width,
				(tile.y - 1) * entity.map.tile.height
			)
		end
		entity.map.changedTiles = {}

		love.graphics.setCanvas(game.layers[entity.map.layer])
		local r,g,b,a = love.graphics.getColor()

		love.graphics.draw(entity.map.canvas,
			math.floor(entity.x + (entity.map.anchored and 0 or game.camera.x)),
			math.floor(entity.y + (entity.map.anchored and 0 or game.camera.y))
		)

		love.graphics.setCanvas(lc)
	end

	function tilemap:renderVersion1_2(entity)
		local lc = love.graphics.getCanvas()
		love.graphics.setCanvas(entity.map.canvas)

		for index, tile in pairs(entity.map.animatedTiles) do
			local tileset = tile.tileset
			love.graphics.setScissor((tile.x - 1) * tileset.tileWidth, (tile.y - 1) * tileset.tileHeight, tileset.tileWidth, tileset.tileHeight)
			love.graphics.clear()
			love.graphics.draw(tileset.sheet,
				tileset.quads[tile.animation[tile.index].quad + 1],
				(tile.x - 1) * tileset.tileWidth,
				(tile.y - 1) * tileset.tileHeight
			)
			love.graphics.setScissor()
		end

		for index, tile in pairs(entity.map.changedTiles) do
			entity.map.tiles[tile.y][tile.x] = tile.id
			local tileset = entity.map.parent.tilesets[1]

			for index, t in pairs(entity.map.parent.tilesets) do
				if tile.id > index then
					tileset = t
				end
			end

			love.graphics.setScissor((tile.x - 1) * tileset.tileWidth, (tile.y - 1) * tileset.tileHeight, tileset.tileWidth, tileset.tileHeight)
			love.graphics.clear()
			love.graphics.draw(tileset.sheet,
				tilesheet.quads[tile.id + 1],
				(tile.x - 1) * tileset.tileWidth,
				(tile.y - 1) * tileset.tileHeight
			)
			love.graphics.setScissor()
		end
		entity.map.changedTiles = {}

		love.graphics.setCanvas(game.layers[entity.map.layer])
		local r,g,b,a = love.graphics.getColor()
		love.graphics.setColor(entity.map.color.r, entity.map.color.g, entity.map.color.b, entity.map.opacity * 255)

		love.graphics.draw(entity.map.canvas,
			math.floor(entity.x + (entity.map.anchored and 0 or game.camera.x)),
			math.floor(entity.y + (entity.map.anchored and 0 or game.camera.y))
		)

		love.graphics.setColor(r, g, b, a)
		love.graphics.setCanvas(lc)
	end

	function tilemap:remove(entity)
		for index, hb in pairs(entity.map.hitboxes) do
			if game:getSystem("physics").world:hasItem(hb) then
				game:getSystem("physics").world:remove(hb)
			end
		end
	end

	return tilemap

end
